#include "zink_device_info.h"
#include "zink_screen.h"

bool
zink_get_physical_device_info(struct zink_screen *screen) 
{
   struct zink_device_info *info = &screen->info;
   bool support_KHR_maintenance1 = false;
   bool support_KHR_maintenance2 = false;
   bool support_KHR_external_memory = false;
   bool support_KHR_external_memory_fd = false;
   bool support_EXT_shader_viewport_index_layer = false;
   bool support_EXT_post_depth_coverage = false;
   bool support_KHR_driver_properties = false;
   bool support_KHR_draw_indirect_count = false;
   bool support_EXT_fragment_shader_interlock = false;
   bool support_KHR_shader_draw_parameters = false;
   bool support_KHR_sampler_mirror_clamp_to_edge = false;
   bool support_EXT_conditional_rendering = false;
   bool support_EXT_transform_feedback = false;
   bool support_EXT_index_type_uint8 = false;
   bool support_EXT_robustness2 = false;
   bool support_EXT_vertex_attribute_divisor = false;
   bool support_EXT_calibrated_timestamps = false;
   bool support_EXT_custom_border_color = false;
   bool support_EXT_blend_operation_advanced = false;
   bool support_EXT_extended_dynamic_state = false;
   bool support_EXT_pipeline_creation_cache_control = false;
   bool support_EXT_shader_stencil_export = false;
#ifdef VK_EXTX_PORTABILITY_SUBSET_EXTENSION_NAME
   bool support_EXTX_portability_subset = false;
#endif
   bool support_KHR_timeline_semaphore = false;
   bool support_EXT_4444_formats = false;
   bool support_EXT_scalar_block_layout = false;
   uint32_t num_extensions = 0;

   // get device memory properties
   vkGetPhysicalDeviceMemoryProperties(screen->pdev, &info->mem_props);

   // enumerate device supported extensions
   if (vkEnumerateDeviceExtensionProperties(screen->pdev, NULL, &num_extensions, NULL) == VK_SUCCESS) {
      if (num_extensions > 0) {
         VkExtensionProperties *extensions = MALLOC(sizeof(VkExtensionProperties) * num_extensions);
         if (!extensions) goto fail;
         vkEnumerateDeviceExtensionProperties(screen->pdev, NULL, &num_extensions, extensions);

         for (uint32_t i = 0; i < num_extensions; ++i) {
            if (!strcmp(extensions[i].extensionName, "VK_KHR_maintenance1")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  info->have_KHR_maintenance1 = true;
               } else {
                  info->have_KHR_maintenance1 = true;
               }
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_maintenance2")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  info->have_KHR_maintenance2 = true;
               } else {
                  info->have_KHR_maintenance2 = true;
               }
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_external_memory")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  info->have_KHR_external_memory = true;
               } else {
                  info->have_KHR_external_memory = true;
               }
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_external_memory_fd")) {
               info->have_KHR_external_memory_fd = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_shader_viewport_index_layer")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  info->have_EXT_shader_viewport_index_layer = true;
               } else {
                  info->have_EXT_shader_viewport_index_layer = true;
               }
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_post_depth_coverage")) {
               info->have_EXT_post_depth_coverage = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_driver_properties")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  support_KHR_driver_properties = true;
               } else {
                  support_KHR_driver_properties = true;
               }
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_draw_indirect_count")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  info->have_KHR_draw_indirect_count = true;
               } else {
                  info->have_KHR_draw_indirect_count = true;
               }
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_fragment_shader_interlock")) {
               support_EXT_fragment_shader_interlock = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_shader_draw_parameters")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  info->have_KHR_shader_draw_parameters = true;
               } else {
                  info->have_KHR_shader_draw_parameters = true;
               }
            }
            if (!strcmp(extensions[i].extensionName, "VK_KHR_sampler_mirror_clamp_to_edge")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  info->have_KHR_sampler_mirror_clamp_to_edge = true;
               } else {
                  info->have_KHR_sampler_mirror_clamp_to_edge = true;
               }
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_conditional_rendering")) {
               support_EXT_conditional_rendering = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_transform_feedback")) {
               support_EXT_transform_feedback = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_index_type_uint8")) {
               support_EXT_index_type_uint8 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_robustness2")) {
               support_EXT_robustness2 = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_vertex_attribute_divisor")) {
               support_EXT_vertex_attribute_divisor = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_calibrated_timestamps")) {
               info->have_EXT_calibrated_timestamps = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_custom_border_color")) {
               support_EXT_custom_border_color = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_blend_operation_advanced")) {
               support_EXT_blend_operation_advanced = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_extended_dynamic_state")) {
               support_EXT_extended_dynamic_state = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_pipeline_creation_cache_control")) {
               support_EXT_pipeline_creation_cache_control = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_shader_stencil_export")) {
               info->have_EXT_shader_stencil_export = true;
            }
         #ifdef VK_EXTX_PORTABILITY_SUBSET_EXTENSION_NAME
   if (!strcmp(extensions[i].extensionName, "VK_EXTX_portability_subset")) {
               support_EXTX_portability_subset = true;
            }
#endif
            if (!strcmp(extensions[i].extensionName, "VK_KHR_timeline_semaphore")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  info->have_KHR_timeline_semaphore = true;
               } else {
                  info->have_KHR_timeline_semaphore = true;
               }
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_4444_formats")) {
               support_EXT_4444_formats = true;
            }
            if (!strcmp(extensions[i].extensionName, "VK_EXT_scalar_block_layout")) {
               if (VK_MAKE_VERSION(1,2,0) >= screen->vk_version) {
                  support_EXT_scalar_block_layout = true;
               } else {
                  support_EXT_scalar_block_layout = true;
               }
            }
         }

         FREE(extensions);
      }
   }

   // get device features
   if (screen->vk_GetPhysicalDeviceFeatures2) {
      // check for device extension features
      info->feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2;

      if (VK_MAKE_VERSION(1,2,0) <= screen->vk_version) {
         info->feats11.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_FEATURES;
         info->feats11.pNext = info->feats.pNext;
         info->feats.pNext = &info->feats11;
         info->have_vulkan11 = true;
      }
      if (VK_MAKE_VERSION(1,2,0) <= screen->vk_version) {
         info->feats12.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES;
         info->feats12.pNext = info->feats.pNext;
         info->feats.pNext = &info->feats12;
         info->have_vulkan12 = true;
      }

   if (support_EXT_fragment_shader_interlock) {
         info->interlock_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_INTERLOCK_FEATURES_EXT;
         info->interlock_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->interlock_feats;
      }
   if (support_EXT_conditional_rendering) {
         info->cond_render_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CONDITIONAL_RENDERING_FEATURES_EXT;
         info->cond_render_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->cond_render_feats;
      }
   if (support_EXT_transform_feedback) {
         info->tf_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT;
         info->tf_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->tf_feats;
      }
   if (support_EXT_index_type_uint8) {
         info->index_uint8_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INDEX_TYPE_UINT8_FEATURES_EXT;
         info->index_uint8_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->index_uint8_feats;
      }
   if (support_EXT_robustness2) {
         info->rb2_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_FEATURES_EXT;
         info->rb2_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->rb2_feats;
      }
   if (support_EXT_vertex_attribute_divisor) {
         info->vdiv_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_FEATURES_EXT;
         info->vdiv_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->vdiv_feats;
      }
   if (support_EXT_custom_border_color) {
         info->border_color_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_FEATURES_EXT;
         info->border_color_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->border_color_feats;
      }
   if (support_EXT_extended_dynamic_state) {
         info->dynamic_state_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_FEATURES_EXT;
         info->dynamic_state_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->dynamic_state_feats;
      }
   if (support_EXT_pipeline_creation_cache_control) {
         info->pipeline_cache_control_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_CREATION_CACHE_CONTROL_FEATURES_EXT;
         info->pipeline_cache_control_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->pipeline_cache_control_feats;
      }
#ifdef VK_EXTX_PORTABILITY_SUBSET_EXTENSION_NAME
   if (support_EXTX_portability_subset) {
         info->portability_subset_extx_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PORTABILITY_SUBSET_FEATURES_EXTX;
         info->portability_subset_extx_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->portability_subset_extx_feats;
      }
#endif
   if (support_EXT_4444_formats) {
         info->format_4444_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_4444_FORMATS_FEATURES_EXT;
         info->format_4444_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->format_4444_feats;
      }
   if (support_EXT_scalar_block_layout) {
         info->scalar_block_layout_feats.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SCALAR_BLOCK_LAYOUT_FEATURES_EXT;
         info->scalar_block_layout_feats.pNext = info->feats.pNext;
         info->feats.pNext = &info->scalar_block_layout_feats;
      }

      screen->vk_GetPhysicalDeviceFeatures2(screen->pdev, &info->feats);
   } else {
      vkGetPhysicalDeviceFeatures(screen->pdev, &info->feats.features);
   }

   // check for device properties
   if (screen->vk_GetPhysicalDeviceProperties2) {
      VkPhysicalDeviceProperties2 props = {};
      props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2;

      if (VK_MAKE_VERSION(1,2,0) <= screen->vk_version) {
         info->props11.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_PROPERTIES;
         info->props11.pNext = props.pNext;
         props.pNext = &info->props11;
      }
      if (VK_MAKE_VERSION(1,2,0) <= screen->vk_version) {
         info->props12.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_PROPERTIES;
         info->props12.pNext = props.pNext;
         props.pNext = &info->props12;
      }

   if (support_KHR_driver_properties) {
         info->driver_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DRIVER_PROPERTIES_KHR;
         info->driver_props.pNext = props.pNext;
         props.pNext = &info->driver_props;
      }
   if (support_EXT_transform_feedback) {
         info->tf_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_PROPERTIES_EXT;
         info->tf_props.pNext = props.pNext;
         props.pNext = &info->tf_props;
      }
   if (support_EXT_robustness2) {
         info->rb2_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_PROPERTIES_EXT;
         info->rb2_props.pNext = props.pNext;
         props.pNext = &info->rb2_props;
      }
   if (support_EXT_vertex_attribute_divisor) {
         info->vdiv_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_PROPERTIES_EXT;
         info->vdiv_props.pNext = props.pNext;
         props.pNext = &info->vdiv_props;
      }
   if (support_EXT_custom_border_color) {
         info->border_color_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_PROPERTIES_EXT;
         info->border_color_props.pNext = props.pNext;
         props.pNext = &info->border_color_props;
      }
   if (support_EXT_blend_operation_advanced) {
         info->blend_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BLEND_OPERATION_ADVANCED_PROPERTIES_EXT;
         info->blend_props.pNext = props.pNext;
         props.pNext = &info->blend_props;
      }
#ifdef VK_EXTX_PORTABILITY_SUBSET_EXTENSION_NAME
   if (support_EXTX_portability_subset) {
         info->portability_subset_extx_props.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PORTABILITY_SUBSET_PROPERTIES_EXTX;
         info->portability_subset_extx_props.pNext = props.pNext;
         props.pNext = &info->portability_subset_extx_props;
      }
#endif

      // note: setting up local VkPhysicalDeviceProperties2.
      screen->vk_GetPhysicalDeviceProperties2(screen->pdev, &props);
   }

   // enable the extensions if they match the conditions given by ext.enable_conds 
   if (screen->vk_GetPhysicalDeviceProperties2) {
   info->have_KHR_maintenance1 |= support_KHR_maintenance1
         ;
   info->have_KHR_maintenance2 |= support_KHR_maintenance2
         ;
   info->have_KHR_external_memory |= support_KHR_external_memory
         ;
   info->have_KHR_external_memory_fd |= support_KHR_external_memory_fd
         ;
   info->have_EXT_shader_viewport_index_layer |= support_EXT_shader_viewport_index_layer
         ;
   info->have_EXT_post_depth_coverage |= support_EXT_post_depth_coverage
         ;
   info->have_KHR_driver_properties |= support_KHR_driver_properties
         ;
   info->have_KHR_draw_indirect_count |= support_KHR_draw_indirect_count
         ;
   info->have_EXT_fragment_shader_interlock |= support_EXT_fragment_shader_interlock
         && (info->interlock_feats.fragmentShaderSampleInterlock)
&& (info->interlock_feats.fragmentShaderPixelInterlock);
   info->have_KHR_shader_draw_parameters |= support_KHR_shader_draw_parameters
         ;
   info->have_KHR_sampler_mirror_clamp_to_edge |= support_KHR_sampler_mirror_clamp_to_edge
         ;
   info->have_EXT_conditional_rendering |= support_EXT_conditional_rendering
         && (info->cond_render_feats.conditionalRendering);
   info->have_EXT_transform_feedback |= support_EXT_transform_feedback
         && (info->tf_feats.transformFeedback);
   info->have_EXT_index_type_uint8 |= support_EXT_index_type_uint8
         && (info->index_uint8_feats.indexTypeUint8);
   info->have_EXT_robustness2 |= support_EXT_robustness2
         && (info->rb2_feats.nullDescriptor);
   info->have_EXT_vertex_attribute_divisor |= support_EXT_vertex_attribute_divisor
         && (info->vdiv_feats.vertexAttributeInstanceRateDivisor);
   info->have_EXT_calibrated_timestamps |= support_EXT_calibrated_timestamps
         ;
   info->have_EXT_custom_border_color |= support_EXT_custom_border_color
         && (info->border_color_feats.customBorderColors);
   info->have_EXT_blend_operation_advanced |= support_EXT_blend_operation_advanced
         && (info->blend_props.advancedBlendNonPremultipliedSrcColor)
&& (info->blend_props.advancedBlendNonPremultipliedDstColor);
   info->have_EXT_extended_dynamic_state |= support_EXT_extended_dynamic_state
         && (info->dynamic_state_feats.extendedDynamicState);
   info->have_EXT_pipeline_creation_cache_control |= support_EXT_pipeline_creation_cache_control
         && (info->pipeline_cache_control_feats.pipelineCreationCacheControl);
   info->have_EXT_shader_stencil_export |= support_EXT_shader_stencil_export
         ;
#ifdef VK_EXTX_PORTABILITY_SUBSET_EXTENSION_NAME
   info->have_EXTX_portability_subset |= support_EXTX_portability_subset
         ;
#endif
   info->have_KHR_timeline_semaphore |= support_KHR_timeline_semaphore
         ;
   info->have_EXT_4444_formats |= support_EXT_4444_formats
         ;
   info->have_EXT_scalar_block_layout |= support_EXT_scalar_block_layout
         && (info->scalar_block_layout_feats.scalarBlockLayout);
   }

   // generate extension list
   num_extensions = 0;

   if (info->have_KHR_maintenance1) {
       info->extensions[num_extensions++] = "VK_KHR_maintenance1";
   } else {
       debug_printf("ZINK: VK_KHR_maintenance1 required!\n");
       goto fail;
   }
   if (info->have_KHR_maintenance2) {
       info->extensions[num_extensions++] = "VK_KHR_maintenance2";
   }
   if (info->have_KHR_external_memory) {
       info->extensions[num_extensions++] = "VK_KHR_external_memory";
   }
   if (info->have_KHR_external_memory_fd) {
       info->extensions[num_extensions++] = "VK_KHR_external_memory_fd";
   }
   if (info->have_EXT_shader_viewport_index_layer) {
       info->extensions[num_extensions++] = "VK_EXT_shader_viewport_index_layer";
   }
   if (info->have_EXT_post_depth_coverage) {
       info->extensions[num_extensions++] = "VK_EXT_post_depth_coverage";
   }
   if (info->have_KHR_driver_properties) {
       info->extensions[num_extensions++] = "VK_KHR_driver_properties";
   }
   if (info->have_KHR_draw_indirect_count) {
       info->extensions[num_extensions++] = "VK_KHR_draw_indirect_count";
   }
   if (info->have_EXT_fragment_shader_interlock) {
       info->extensions[num_extensions++] = "VK_EXT_fragment_shader_interlock";
   }
   if (info->have_KHR_shader_draw_parameters) {
       info->extensions[num_extensions++] = "VK_KHR_shader_draw_parameters";
   }
   if (info->have_KHR_sampler_mirror_clamp_to_edge) {
       info->extensions[num_extensions++] = "VK_KHR_sampler_mirror_clamp_to_edge";
   }
   if (info->have_EXT_conditional_rendering) {
       info->extensions[num_extensions++] = "VK_EXT_conditional_rendering";
   }
   if (info->have_EXT_transform_feedback) {
       info->extensions[num_extensions++] = "VK_EXT_transform_feedback";
   }
   if (info->have_EXT_index_type_uint8) {
       info->extensions[num_extensions++] = "VK_EXT_index_type_uint8";
   }
   if (info->have_EXT_robustness2) {
       info->extensions[num_extensions++] = "VK_EXT_robustness2";
   }
   if (info->have_EXT_vertex_attribute_divisor) {
       info->extensions[num_extensions++] = "VK_EXT_vertex_attribute_divisor";
   }
   if (info->have_EXT_calibrated_timestamps) {
       info->extensions[num_extensions++] = "VK_EXT_calibrated_timestamps";
   }
   if (info->have_EXT_custom_border_color) {
       info->extensions[num_extensions++] = "VK_EXT_custom_border_color";
   }
   if (info->have_EXT_blend_operation_advanced) {
       info->extensions[num_extensions++] = "VK_EXT_blend_operation_advanced";
   }
   if (info->have_EXT_extended_dynamic_state) {
       info->extensions[num_extensions++] = "VK_EXT_extended_dynamic_state";
   }
   if (info->have_EXT_pipeline_creation_cache_control) {
       info->extensions[num_extensions++] = "VK_EXT_pipeline_creation_cache_control";
   }
   if (info->have_EXT_shader_stencil_export) {
       info->extensions[num_extensions++] = "VK_EXT_shader_stencil_export";
   }
#ifdef VK_EXTX_PORTABILITY_SUBSET_EXTENSION_NAME
   if (info->have_EXTX_portability_subset) {
       info->extensions[num_extensions++] = "VK_EXTX_portability_subset";
   }
#endif
   if (info->have_KHR_timeline_semaphore) {
       info->extensions[num_extensions++] = "VK_KHR_timeline_semaphore";
   }
   if (info->have_EXT_4444_formats) {
       info->extensions[num_extensions++] = "VK_EXT_4444_formats";
   }
   if (info->have_EXT_scalar_block_layout) {
       info->extensions[num_extensions++] = "VK_EXT_scalar_block_layout";
   }

   info->num_extensions = num_extensions;

   return true;

fail:
   return false;
}
